/* $Id: compat-generic.c,v 1.1 2005/03/18 12:44:29 larsv Exp $
 *
 * ETRAX FS IOP USB 1.1 Host Controller
 *
 * Generic compatibility layer
 *
 * Copyright (C) 2004, 2005  Axis Communications AB
 *
 * Author: Lars Viklund <larsv@axis.com>
 */

#include "compat.h"
#include <asm/arch/hwregs/config_defs.h>
#include <asm/arch/hwregs/strmux_defs.h>
#include <asm/arch/hwregs/pinmux_defs.h>
#include "iop-usb-int.h"

#define PINMUX_REG_ADDR(port) \
	(REG_ADDR(pinmux, regi_pinmux, rw_pb_iop) + 8*(port))

/* allocate DMA channel for IOP use */
int iop_request_dma(int dmc, int dma_inst, const char *device_id)
{
	reg_strmux_rw_cfg strmux_cfg = REG_RD(strmux, regi_strmux, rw_cfg);
	switch (dmc) {
	case 0:
		switch (dma_inst) {
		case regi_dma2:
			strmux_cfg.dma2 = regk_strmux_iop0;
			break;
		case regi_dma3:
			strmux_cfg.dma3 = regk_strmux_iop0;
			break;
		default:
			return -1;
		}
		break;
	case 1:
		switch (dma_inst) {
		case regi_dma4:
			strmux_cfg.dma4 = regk_strmux_iop1;
			break;
		case regi_dma5:
			strmux_cfg.dma5 = regk_strmux_iop1;
			break;
		default:
			return -1;
		}
		break;
	default:
		return -1;
	}
	REG_WR(strmux, regi_strmux, rw_cfg, strmux_cfg);
	return 0;
}

/* deallocate DMA channel */
int iop_free_dma(int dma_inst)
{
	reg_strmux_rw_cfg strmux_cfg = REG_RD(strmux, regi_strmux, rw_cfg);
	switch (dma_inst) {
	case regi_dma2:
		strmux_cfg.dma2 = regk_strmux_off;
		break;
	case regi_dma3:
		strmux_cfg.dma3 = regk_strmux_off;
		break;
	case regi_dma4:
		strmux_cfg.dma4 = regk_strmux_off;
		break;
	case regi_dma5:
		strmux_cfg.dma5 = regk_strmux_off;
		break;
	default:
		return -1;
	}
	REG_WR(strmux, regi_strmux, rw_cfg, strmux_cfg);
	return 0;
}

/* allocate pins for IOP use */
int iop_pinmux_alloc(int port, int first_pin, int last_pin)
{
	int val;
	unsigned int i;

	val = REG_READ(int, PINMUX_REG_ADDR(port));
	for (i = first_pin; i <= last_pin; i++)
		val |= (1 << i);
	REG_WRITE(int, PINMUX_REG_ADDR(port), val);
	return 0;
}

/* deallocate pins */
int iop_pinmux_dealloc(int port, int first_pin, int last_pin)
{
	int val;
	unsigned int i;

	val = REG_READ(int, PINMUX_REG_ADDR(port));
	for (i = first_pin; i <= last_pin; i++)
		val &= ~(1 << i);
	REG_WRITE(int, PINMUX_REG_ADDR(port), val);
	return 0;
}

/* configure pin multiplexing for internal PHY */
void iop_usb_phy(int port)
{
	reg_pinmux_rw_usb_phy usb_phy = { 
		.en_usb0 = regk_pinmux_no,
		.en_usb1 = regk_pinmux_no
	};
	switch (port) {
	case 0:
		usb_phy.en_usb0 = regk_pinmux_yes;
		break;
	case 1:
		usb_phy.en_usb1 = regk_pinmux_yes;
		break;
	default:
		break;
	}
	REG_WR(pinmux, regi_pinmux, rw_usb_phy, usb_phy);
}

/* start clock for IOP */
void iop_start_clock(void)
{
	reg_config_rw_clk_ctrl clk_ctrl;

	clk_ctrl = REG_RD(config, regi_config, rw_clk_ctrl);
	clk_ctrl.iop = regk_config_yes;
	REG_WR(config, regi_config, rw_clk_ctrl, clk_ctrl);
}
