/* $Id: compat-linux.c,v 1.2 2005/05/02 11:31:36 larsv Exp $
 *
 * ETRAX FS IOP USB 1.1 Host Controller
 *
 * Linux kernel compatibility layer
 *
 * Copyright (C) 2004, 2005  Axis Communications AB
 *
 * Author: Lars Viklund <larsv@axis.com>
 */

#include "compat.h"
#include <linux/errno.h>
#include <asm/arch/hwregs/config_defs.h>
#include <asm/arch/hwregs/pinmux_defs.h>
#include "iop-usb-int.h"

/* allocate DMA channel for IOP use */
int iop_request_dma(int dmc, int dma_inst, const char *device_id)
{
	unsigned int dmanr;
	enum dma_owner dma_owner;

	switch (dmc) {
	case 0:
		dma_owner = dma_iop0;
		break;
	case 1:
		dma_owner = dma_iop1;
		break;
	default:
		return -1;
	}
	switch (dma_inst) {
	case regi_dma2:
		dmanr = 2;
		break;
	case regi_dma3:
		dmanr = 3;
		break;
	case regi_dma4:
		dmanr = 4;
		break;
	case regi_dma5:
		dmanr = 5;
		break;
	default:
		return -1;
	}
	return crisv32_request_dma(dmanr, device_id, 0, 12000000/8, dma_owner);
}

/* deallocate DMA channel */
int iop_free_dma(int dma_inst)
{
	unsigned int dmanr;

	switch (dma_inst) {
	case regi_dma2:
		dmanr = 2;
		break;
	case regi_dma3:
		dmanr = 3;
		break;
	case regi_dma4:
		dmanr = 4;
		break;
	case regi_dma5:
		dmanr = 5;
		break;
	default:
		return -1;
	}
	crisv32_free_dma(dmanr);
	return 0;
}

/* allocate pins for IOP use */
int iop_pinmux_alloc(int port, int first_pin, int last_pin)
{
	return crisv32_pinmux_alloc(port, first_pin, last_pin, pinmux_iop);
}

/* deallocate pins */
int iop_pinmux_dealloc(int port, int first_pin, int last_pin)
{
	return crisv32_pinmux_dealloc(port, first_pin, last_pin);
}

/* configure pin multiplexing for internal PHY */
void iop_usb_phy(int port)
{
	reg_pinmux_rw_usb_phy usb_phy = { 
		.en_usb0 = regk_pinmux_no,
		.en_usb1 = regk_pinmux_no
	};
	switch (port) {
	case 0:
		usb_phy.en_usb0 = regk_pinmux_yes;
		break;
	case 1:
		usb_phy.en_usb1 = regk_pinmux_yes;
		break;
	default:
		break;
	}
	REG_WR(pinmux, regi_pinmux, rw_usb_phy, usb_phy);
}

/* start clock for IOP */
void iop_start_clock(void)
{
	reg_config_rw_clk_ctrl clk_ctrl;

	clk_ctrl = REG_RD(config, regi_config, rw_clk_ctrl);
	clk_ctrl.iop = regk_config_yes;
	REG_WR(config, regi_config, rw_clk_ctrl, clk_ctrl);
}
